use std::{fs, path::PathBuf};

use log::debug;
use sqlx::{migrate::MigrateDatabase, Connection, Sqlite, SqliteConnection, SqlitePool};

#[derive(sqlx::FromRow, Debug)]
pub struct Sheet {
    name: String,
    // #[sqlx(from = "String")]
    path: String,
    // #[sqlx(from = "i64")]
    file_size: i32,
    file_hash: String,
}

#[derive(sqlx::FromRow)]
pub struct Composer {
    name: String,
    id: u32,
}

impl Sheet {
    pub fn new_debug() -> Self {
        Sheet {
            name: "Hello world".to_string(),
            path: "This/is/my/path".into(),
            file_size: 42,
            file_hash: "h4sh".to_string(),
        }
    }

    pub fn verify_path(&self) -> std::io::Result<bool> {
        // First compare file size since it is faster than hashing
        // let file_size = fs::metadata(&self.path)?.len();
        // if file_size == self.file_size {
        //     let file_content = fs::read(&self.path)?;
        //     let file_hash = blake3::hash(&file_content);
        //     if file_hash.to_string() == self.file_hash {
        //         return Ok(true);
        //     }
        // }

        Ok(false)
    }
}

pub struct Database {
    connection: SqlitePool,
}

impl Database {
    pub async fn setup(database_path: &str) -> sqlx::Result<Self> {
        let connection = Database::create_connection(database_path).await?;
        sqlx::migrate!("./db-migrations").run(&connection).await?;
        Ok(Database { connection })
    }

    async fn create_connection(database_path: &str) -> sqlx::Result<SqlitePool> {
        if !Sqlite::database_exists(database_path)
            .await
            .unwrap_or(false)
        {
            match Sqlite::create_database(database_path).await {
                Ok(_) => debug!("Database created"),
                Err(e) => {
                    debug!("Error creating database: {}", e);
                    return Err(e);
                }
            }
        }

        let connection = SqlitePool::connect(database_path).await?;
        debug!("Connected to database");
        Ok(connection)
    }

    pub async fn insert_sheet(&self, sheet: Sheet) -> sqlx::Result<()> {
        sqlx::query!(
            "
                INSERT INTO sheets (name, path, file_size, file_hash)
                VALUES ($1, $2, $3, $4)
            ",
            sheet.name,
            sheet.path,
            sheet.file_size,
            sheet.file_hash,
        )
        .execute(&self.connection)
        .await
        .map(|_| ())
    }

    pub async fn fetch_all_sheets(&self) -> sqlx::Result<Vec<Sheet>> {
        sqlx::query_as::<_, Sheet>("SELECT * FROM sheets")
            .fetch_all(&self.connection)
            .await
    }
}
